//This code solves and simulates a stochastic realization from the model described in 
//Christiano: 'Simple New Keynesian Model Without Capital: Implications of Networks'.

//The output is two rows of three figures each: in the top row, gam = 1/2, bottom gam = 1
//Each figure contains two graphs: one in which first order perturbation was used and the other where second order 
//perturbation was used

//Note, (i) model with gamma = 1/2 is about twice as volatile as with gamma = 1, illustrating the 'multiplier' effect
// on technology reported in the network literature.
// (ii) first and second order perturbation roughly the same, suggesting first order is enough...except in case of 
//      inflation distortion which is understated by first order perturbation.
// (iii) inflation distortion bigger with gamma=1/2 

//Exercises: try the following one-at-a-time changes:
//pibar = 1.0 and note that to first order there is no inflation distortion. For proof see p. 38 in handout.
//pibar = 1.01 - 4 percent annual inflation and distortions bigger. Losses greater
//eps = 7.7 - more competition. Increase in inflation distorions
//nu = 0,  no subsidy to remove the effects of steady state monopoly power. Distortions (1-chi) become very large
//         when there are networks. In absence of networks there are no direct monoply distortions in chi.
//         This is another important implication of networks, beyond their implications for the cost of inflation.
//alpha = 0. This corresponds to "Taylor's Taylor rule", which is a static equation. When alpha = 0, you should find that
//           that there is indeterminacy: a form of multiple equilibria. This is because there are multiple self-fullfilling 
//           expected inflation rates, despite the fact that the Taylor principle is satisfied (i.e., phi_pie = 1.5 > 0).
//           The problem is that when there are networks and a full working capital channel, gam=1/2, then there is 
//           a lot of borrowing in the economy: two times the amount of GDP. Thus, suppose there is a one percent rise in 
//           expected inflation and the monetary authority responds by raising the interest rate by more than one
//           percent so that the expected real rate rises. The 'demand channel' implies that demand falls. Other things
//           the same, this slows down the economy
//           and reduces inflation. If nothing else happened, this cannot happen in a rational expectations equilibrium
//           because it contradicts agents' assumption that inflation will rise. But, a 'supply channel' operating 
//           through firm costs directly drives inflation up
//           in the presence of working capital (psi>0). When there are networks, this channel is big and can overwhelm the
//           demand channel. Thus, the operation of the Taylor principle, rather than reigning in inflation, can produce the
//           consequence that a rise in the interest rate raises inflation. When alpha is big, then we get uniqueness
//           of equilibrium back again by making the demand channel stronger. When alpha is big, then an increase 
//           in the interest rate is expected to be persistent, so that there is an increase in the long term interest
//           rate. Consumption is a function of the long term interest rate. This is why it is that when alpha is 
//           large there is uniqueness of equilibrium again. Note, however, that uniqueness
//           of equilibrium is more tenuous, when there are networks.
// Replace piebar in the monetary policy equation with piebar(+1) and verify that the indeterminacy consequences of 
//           changing the value of alpha from zero to the benchmark value is unaffected. The usual economic intuition about 
//           the consequences of the Taylor principle for indeterminacy is expressed in terms of piebar(+1), but the results
//           are not sensitive to that.
// Replace chi_check=pst_SS^(1/gam); in the initval statement with chi_checkect0.1; which is an error. Since overall you're
//           still giving Dynare a pretty good guess about the steady state, it can move quickly to fix it after 
//           detecting it. So, Dynare doesn't need a perfect guess to find the steady state, but it does need a pretty
//           good guess.

// Endogenous variables
var R pibar pstar K F N C Y I mu chi s;

var chi_check chi_check_no_monopoly a tau;

// Innovations
varexo eps_A eps_tau;

// PARAMETERS
parameters R_SS pibar_SS gam eps phi rho nu theta alpha rholambda phi_pie beta psiI psiN;

phi = 1;
rho = 0.9;
pibar_SS=1.0062;%2.5 percent inflation, at an annual rate
//pibar_SS=1.01;
//pibar_SS=1;
psiI=1;
psiN=1;
//psiI=0;
gam=.5;
beta=1.03^(-.25);
theta=0.75;
eps=6;
//eps=7.7;
phi=1;

%this gives the value of the subsidy, nu, which sets the cost of working equal to the marginal product of labor
%in the steady state, it also sets the materials to gross output ratio to 1-gamma.
xx=(eps-1)/(eps*(1-phi+phi*(1/beta)))*((1-beta*theta*pibar_SS^eps)/(1-theta*pibar_SS^eps))*((1-theta*pibar_SS^(eps-1))/((1-beta*theta*pibar_SS^(eps-1))));
nu=1-xx;
//nu=0;
alpha = 0.8;
//alpha = 0;
rholambda = 0.9;
phi_pie = 1.5;

[RR_SS,s_SS,pst_SS,mu_SS,C_SS,Y_SS,N_SS,TFP_dist_SS,K_SS,F_SS,I_SS,chi_SS] = ...
    ss_state(pibar_SS,beta,theta,eps,gam,psiI,psiN,nu,phi);

R_SS=RR_SS;
chi_check_SS=(pst_SS*( (1-(1-gam)/pst_SS)/gam )^gam*( (((1-gam)/pst_SS)/(1-gam))^(1-gam) ))^(1/gam);

model;
# cst= (gam^gam*(1-gam)^(1-gam))^(1/gam);

//1 numbers are equation numbers in handout
(eps/(eps-1))*s*Y/C + beta*theta*(pibar(+1)^eps)*K(+1) - K;
//2
Y/C + (pibar(+1)^(eps-1))*beta*theta*F(+1) - F;
//3
F*( ((1-theta*pibar^(eps-1))/(1-theta))^(1/(1-eps)) ) - K;
//4
1/pstar - ((1-theta)*(((1-theta*(pibar)^(eps-1))/(1-theta))^(eps/(eps-1))) + theta*(pibar^eps)/pstar(-1));
//6
Y=pstar*exp(a)*N^gam*I^(1-gam);
//7
C+I=Y;
//8
I=mu*Y/pstar;
//9
s=(1-nu)*(((1-psiI+psiI*R)/(1-gam))^(1-gam))*((1-psiN+psiN*R)*exp(tau)*C*N^phi/gam)^gam*exp(-a);
//10
mu=(1-gam)*s/((1-nu)*(1-psiI+psiI*R));
//11
chi=(pstar*( (1-mu/pstar)/gam )^gam*( ((mu/pstar)/(1-gam))^(1-gam) ))^(1/gam);
//5
1/C - (beta/C(+1))*(R/pibar(+1));

chi_check_no_monopoly=(pstar*( (1-(1-gam)/pstar)/gam )^gam*( (((1-gam)/pstar)/(1-gam))^(1-gam) ))^(1/gam);

chi_check=(pstar)^(1/gam);

//monetary policy rule
R/R_SS = ((R(-1)/R_SS)^alpha) * exp( (1-alpha) *  phi_pie*(pibar-pibar_SS) );
//
a=.95*a(-1)+eps_A;
tau = rholambda*tau(-1)+eps_tau;

end;

initval;
R=R_SS;
tau=0; 
pibar=pibar_SS; 
a=0; 
pstar=pst_SS; 
K=K_SS; 
F=F_SS;
N=N_SS;
C=C_SS;
I=I_SS;
s=s_SS;
Y=Y_SS;
chi=chi_SS;
mu=mu_SS;
chi_check=pst_SS^(1/gam);
chi_check_no_monopoly=chi_check_SS;
end;

steady;
resid;
check;

shocks;
var eps_A;
stderr .01;
var eps_tau;
stderr .01;
end;

set_dynare_seed(1);
stoch_simul(order=2,nograph,pruning,periods=200);
sim2=oo_.endo_simul;

chi_check_a=sim2(13,:);%this is p-star^(1/gam)
chi_gam_half2=sim2(11,:);
no_monopoly_gam_half2=sim2(14,:);%this is chi with mu replaced with 1-gamma
check_gam_half2=100*(no_monopoly_gam_half2-chi_check_a)./no_monopoly_gam_half2;

pstar_gam_half2=sim2(3,:);
infl_gam_half2=sim2(2,:);
C_gam_half2=100*(sim2(7,:)-C_SS)/C_SS;%this is 100 times the deviation between logC and its steady state value

set_dynare_seed('reset');
stoch_simul(order=1,nograph,periods=200);
sim1=oo_.endo_simul;
chi_gam_half1=sim1(11,:);
pstar_gam_half1=sim1(3,:);
C_gam_half1=100*(sim1(7,:)-C_SS)/C_SS;
infl_gam_half1=sim1(2,:);


% change the value of gam, in the 3rd element of params
gam=0.999;
M_.params( 3 ) = .999;

[RR_SS,s_SS,pst_SS,mu_SS,C_SS,Y_SS,N_SS,TFP_dist_SS,K_SS,F_SS,I_SS,chi_SS] = ...
    ss_state(pibar_SS,beta,theta,eps,gam,psiI,psiN,nu,phi);
chi_check_SS=(pst_SS*( (1-(1-gam)/pst_SS)/gam )^gam*( (((1-gam)/pst_SS)/(1-gam))^(1-gam) ))^(1/gam);

initval;
R=R_SS;
tau=0; 
pibar=pibar_SS; 
a=0; 
pstar=pst_SS; 
K=K_SS; 
F=F_SS;
N=N_SS;
C=C_SS;
I=I_SS;
s=s_SS;
Y=Y_SS;
chi=chi_SS;
mu=mu_SS;
chi_check=pst_SS^(1/gam);
chi_check_no_monopoly=chi_check_SS;
end;

set_dynare_seed('reset');
stoch_simul(order=2,nograph,pruning,periods=200);
sim2=oo_.endo_simul;
chi_gam_one2=sim2(11,:);
pstar_gam_one2=sim2(3,:);
C_gam_one2=100*(sim2(7,:)-C_SS)/C_SS;
infl_gam_one2=sim2(2,:);

set_dynare_seed('reset');
stoch_simul(order=1,nograph,periods=200);
sim1=oo_.endo_simul;
chi_gam_one1=sim1(11,:);
pstar_gam_one1=sim1(3,:);
C_gam_one1=100*(sim1(7,:)-C_SS)/C_SS;
infl_gam_one1=sim1(2,:);

ia=2;
ib=3;

subplot(ia,ib,1)
tt=1:length(chi_gam_one1);
plot(tt,100*(1-chi_gam_half1),tt,100*(1-chi_gam_half2),'*-')
legend('first order','second order')
ylabel('percent')
title('Gamma = 1/2, 100*(1-chi)')
axis tight

subplot(ia,ib,ib+1)
plot(tt,100*(1-chi_gam_one1),tt,100*(1-chi_gam_one2),'*-')
legend('first order','second order')
ylabel('level')
title('Gamma = 1, 100*(1-chi)')
axis tight

subplot(ia,ib,2)
plot(tt,C_gam_half1,tt,C_gam_half2,'*-')
legend('first order','second order')
ylabel('perc dev from ss')
title('Gamma = 1/2, C')
axis tight

subplot(ia,ib,ib+2)
plot(tt,C_gam_one1,tt,C_gam_one2,'*-')
legend('first order ','second order')
ylabel('perc dev from ss')
title('Gamma = 1, C')
axis tight

subplot(ia,ib,3)
plot(tt,400*(infl_gam_half1-1),tt,400*(infl_gam_half2-1),'*-')
legend('first order','second order')
ylabel('APR')
title('Gamma = 1/2, \pi_{t}')
axis tight

subplot(ia,ib,ib+3)
plot(tt,400*(infl_gam_one1-1),tt,400*(infl_gam_one2-1),'*-')
legend('first order ','second order')
ylabel('APR')
title('Gamma = 1, \pi_{t}')
axis tight


sC_gam_one1=std(C_gam_one1);sC_gam_half1=std(C_gam_half1);sC_gam_one2=std(C_gam_one2);sC_gam_half2=std(C_gam_half2)
fprintf('sC_gam_one1 = %5.3f, sC_gam_half1 = %5.3f, sC_gam_one2 = %5.3f, sC_gam_half2 = %5.3f\n', ...
    sC_gam_one1,sC_gam_half1,sC_gam_one2,sC_gam_half2)
fprintf(' mean percent deviation between pstar^(1/gam) and chi = %5.3f\n',mean(check_gam_half2))
orient landscape
%print -dpdf materials_more_competition

